@@ -13,7 +13,7 @@ use Math::MPC qw();
 
 use POSIX qw(ULONG_MAX LONG_MIN);
 
-our $VERSION = '0.17';
+our $VERSION = '0.18';
 our ($ROUND, $PREC);
 
 BEGIN {
@@ -383,7 +383,7 @@ use overload
     }
 }
 
-# Converts a string into an mpq object
+# Create and return a new {GMP*, MPFR, MPC} object, given a base-10 numerical string
 sub _str2obj {
     my ($s) = @_;
 
@@ -497,7 +497,7 @@ sub _str2obj {
     eval { Math::GMPz::Rmpz_init_set_str($s, 10) } // goto &_nan;
 }
 
-# Parse a base-10 string as a base-10 fraction
+# Parse a given decimal expansion string as a base-10 fraction
 sub _str2frac {
     my ($str) = @_;
 
@@ -511,8 +511,7 @@ sub _str2frac {
         $sign = '';
     }
 
-    my $i;
-    if (($i = index($str, 'e')) != -1) {
+    if ((my $i = index($str, 'e')) != -1) {
 
         my $exp = substr($str, $i + 1);
 
@@ -535,41 +534,40 @@ sub _str2frac {
             }
         }
 
-        my $numerator   = "$before$after";
-        my $denominator = "1";
+        my $numerator = "$sign$before$after";
 
-        if ($exp < 1) {
-            $denominator .= '0' x (CORE::abs($exp) + CORE::length($after));
-        }
-        else {
-            my $diff = ($exp - CORE::length($after));
-            if ($diff >= 0) {
-                $numerator .= '0' x $diff;
-            }
-            else {
-                my $s = "$before$after";
-                substr($s, $exp + CORE::length($before), 0, '.');
-                return _str2frac("$sign$s");
-            }
+        if ($exp < 0) {
+            return ("$numerator/1" . ('0' x (CORE::abs($exp) + CORE::length($after))));
         }
 
-        "$sign$numerator/$denominator";
+        my $diff = ($exp - CORE::length($after));
+
+        if ($diff >= 0) {
+            return ($numerator . ('0' x $diff));
+        }
+
+        my $s = "$before$after";
+        substr($s, $exp + CORE::length($before), 0, '.');
+        return _str2frac("$sign$s");
     }
-    elsif (($i = index($str, '.')) != -1) {
+
+    if ((my $i = index($str, '.')) != -1) {
         my ($before, $after) = (substr($str, 0, $i), substr($str, $i + 1));
-        if (($after =~ tr/0//) == CORE::length($after)) {
+
+        if ($after == 0) {
             return "$sign$before";
         }
-        $sign . ("$before$after/1" =~ s/^0+//r) . ('0' x CORE::length($after));
-    }
-    else {
-        "$sign$str";
+
+        return ($sign . "$before$after/1" . ('0' x CORE::length($after)));
     }
+
+    return "$sign$str";
 }
 
 #
 ## MPZ
 #
+
 sub _mpz2mpq {
     my $r = Math::GMPq::Rmpq_init();
     Math::GMPq::Rmpq_set_z($r, $_[0]);
@@ -901,7 +899,7 @@ sub new {
             my $r = Math::GMPq::Rmpq_init();
             eval { Math::GMPq::Rmpq_set_str($r, $num, $int_base); 1 } // goto &nan;
 
-            if (Math::GMPq::Rmpq_get_str($r, 10) !~ m{^\s*[-+]?[0-9]+\s*/\s*[-+]?[1-9]+[0-9]*\s*\z}) {
+            if (Math::GMPq::Rmpq_get_str($r, 10) !~ m{^\s*[-+]?[0-9]+\s*(?:/\s*[-+]?[1-9]+[0-9]*\s*)?\z}) {
                 goto &nan;
             }
 
@@ -1386,25 +1384,24 @@ sub int {    # used in overloading
 
 sub rat ($) {
     my ($x) = @_;
+
     if (ref($x) eq __PACKAGE__) {
         ref($$x) eq 'Math::GMPq' && return $x;
-        bless \(_any2mpq($$x) // (goto &nan));
+        return bless \(_any2mpq($$x) // (goto &nan));
     }
-    else {
 
-        # Parse a decimal number as an exact fraction
-        if ("$x" =~ /^([+-]?+(?=\.?[0-9])[0-9_]*+(?:\.[0-9_]++)?(?:[Ee](?:[+-]?+[0-9_]+))?)\z/) {
-            my $frac = _str2frac(lc($1));
-            my $q    = Math::GMPq::Rmpq_init();
-            Math::GMPq::Rmpq_set_str($q, $frac, 10);
-            Math::GMPq::Rmpq_canonicalize($q) if (index($frac, '/') != -1);
-            return bless \$q;
-        }
-
-        my $r = _star2obj($x);
-        ref($r) eq 'Math::GMPq' && return bless \$r;
-        bless(\_any2mpq($r) // goto &nan);
+    # Parse a decimal number as an exact fraction
+    if ("$x" =~ /^([+-]?+(?=\.?[0-9])[0-9_]*+(?:\.[0-9_]++)?(?:[Ee](?:[+-]?+[0-9_]+))?)\z/) {
+        my $frac = _str2frac(lc($1));
+        my $q    = Math::GMPq::Rmpq_init();
+        Math::GMPq::Rmpq_set_str($q, $frac, 10);
+        Math::GMPq::Rmpq_canonicalize($q) if (index($frac, '/') != -1);
+        return bless \$q;
     }
+
+    my $r = _star2obj($x);
+    ref($r) eq 'Math::GMPq' && return bless \$r;
+    bless \(_any2mpq($r) // (goto &nan));
 }
 
 #
@@ -4296,10 +4293,58 @@ sub rat_approx ($) {
 
 sub digits ($;$) {
     my ($x, $y) = @_;
-    my $str = as_int($x, $y) // return ();
-    my @digits = split(//, $str);
-    shift(@digits) if $digits[0] eq '-';
-    (@digits);
+
+    $x = _star2mpz($x) // return;
+    $y //= 10;
+
+    if (!ref($y) and CORE::int($y) eq $y and $y > 1 and $y < ULONG_MAX) {
+
+        if ($y <= 10) {
+            my @digits = split(//, scalar reverse scalar Math::GMPz::Rmpz_get_str($x, $y));
+            pop(@digits) if $digits[-1] eq '-';
+            return @digits;
+        }
+
+        if ($y == 16) {
+            my @digits = split(//, scalar reverse scalar Math::GMPz::Rmpz_get_str($x, $y));
+            pop(@digits) if $digits[-1] eq '-';
+            return map { hex($_) } @digits;
+        }
+    }
+
+    $y = _star2mpz($y) // return;
+
+    # Not defined for y <= 1
+    if (Math::GMPz::Rmpz_cmp_ui($y, 1) <= 0) {
+        return;
+    }
+
+    # Return faster when y <= 10
+    if (Math::GMPz::Rmpz_cmp_ui($y, 10) <= 0) {
+        my @digits = split(//, scalar reverse scalar Math::GMPz::Rmpz_get_str($x, Math::GMPz::Rmpz_get_ui($y)));
+        pop(@digits) if $digits[-1] eq '-';
+        return @digits;
+    }
+
+    my @digits;
+    my $t = Math::GMPz::Rmpz_init_set($x);
+
+    my $sgn = Math::GMPz::Rmpz_sgn($t);
+
+    if ($sgn == 0) {
+        return (zero());
+    }
+    elsif ($sgn < 0) {
+        Math::GMPz::Rmpz_abs($t, $t);
+    }
+
+    while (Math::GMPz::Rmpz_sgn($t) > 0) {
+        my $m = Math::GMPz::Rmpz_init();
+        Math::GMPz::Rmpz_divmod($t, $m, $t, $y);
+        push @digits, bless \$m;
+    }
+
+    return @digits;
 }
 
 1;    # End of Math::AnyNum
